source("common.R")

## Needs to be run after "budgets.R"

## Calcule, en NPV à t=0, pour chacune des n.sims.trunc simulations,
## et pour les différentes valeurs de n et r:
## la différence, à t=h.years, entre la borne mobile de solvabilité éternelle,
## et la valeur effectivement atteinte par le fonds, en supposant F_0 = 0
financing.needs.sims <- function(sims, h.years = 50, n.sims.trunc = length(sims))
{
  if (!inherits(sims, "sims"))
    stop("You must provide an argument of class sims.")

  if (n.sims.trunc < 1 || n.sims.trunc > length(sims))
    stop("Incorrect n.sims.trunc.")

  if (h.years > attr(sims, "h.years"))
    stop("The specified h.years is bigger than the years actually simulated.")

  d <- 1

  na <- length(years.list)
  nb <- length(r.annual.list)

  ## Un tableau avec des suites de R^t
  Rs <- numeric()
  for (b in 1:nb)
    Rs <- rbind(Rs, (1/(1 + r.annual.list[b]))^(0:h.years))

  ## La liste double entrée des polynômes K(RL)
  K <- list()
  for (b in 1:nb)
    {
      K[[b]] <- list()
      for (a in 1:na)
        K[[b]][[a]] <- c(rep(1, d-1), (years.list[a]:1) / years.list[a]) * (Rs[b, 1:d-1+years.list[a]])
    }

  ## Un tableau double entrée avec les R*K(R)
  R.K.R <- array(0, c(nb, na))
  for (a in 1:na)
    for (b in 1:nb)
      R.K.R[b, a] <- Rs[b, 2] * K.R(years.list[a], d, r.annual.list[b])

  budgets <- array(0, c(nb, na, n.sims.trunc))

  for(i in 1:n.sims.trunc)
    {
      sim <- window(sims[[i]], end = start(sims[[i]]) + h.years)

      ## Calcul de Dt
      Dt.nlag.max <- max(years.list) + d - 1
      sim.rd <- rev(diff(sim[(h.years+2-Dt.nlag.max):(h.years+1)]))
      Dt <- list()
      for (b in 1:nb)
        Dt[[b]] <- c(0, cumsum(sim.rd / Rs[b, 1:(Dt.nlag.max - 1)]))

      for (a in 1:na)
        {
          x <- ma.annual(sim, years.list[a], d, diff = TRUE)
          x[1] <- 0

          Dt.nlag <- years.list[a] + d - 1		# Nombre de termes de Dt qu'il va falloir inclure dans le calcul
          ## Égal à length(K[[b]][[a]])

          for (b in 1:nb)
            {
              Ft <- sum(x*Rs[b, ])

              bound <- Rs[b, h.years + 1] * (R.K.R[b, a] * sim[h.years + 1] - K[[b]][[a]] %*% Dt[[b]][1:Dt.nlag])

              budgets[b, a, i] <- bound - Ft
            }
        }
    }

  dimnames(budgets) <- list(r.annual.list, years.list, NULL)
  structure(budgets, class = "budgets", fit = attr(sims, "fit"), h.years = h.years)
}


surplus.onecomm <- function(budgets, financing.needs, quant = 0.9, ref.price = get.ref.price.budgets(budgets), n.sims.trunc = dim(financing.needs)[3])
{
  if (!inherits(budgets, "budgets"))
    stop("budgets must be of class budgets.")
  if (!inherits(financing.needs, "budgets"))
    stop("financing.needs must be of class budgets.")
  if ((dim(budgets)[1] != dim(financing.needs)[1]) || (dim(budgets)[2] != dim(financing.needs)[2]))
    stop("Dims don't match.")

  price.quants <- array(0, c(dim(budgets)[1], dim(budgets)[2]))
  for (i in 1:dim(budgets)[1])
    for (j in 1:dim(budgets)[2])
      price.quants[i, j] <- as.numeric(quantile(budgets[i, j, 1:n.sims.trunc], quant))

  surplus <- array(0, c(dim(budgets)[1], dim(budgets)[2], n.sims.trunc))

  for (i in 1:n.sims.trunc)
    {
      x <- (budgets[,,i] < price.quants)
      y <- price.quants - financing.needs[,,i]
      surplus[,,i] <- pmax(x * y, 0)
    }

  results <- array(0, c(dim(budgets)[1], dim(budgets)[2], 2))

  for (i in 1:dim(budgets)[1])
    for (j in 1:dim(budgets)[2])
      {
        results[i, j, 1] <- median(surplus[i,j,]) / ref.price
        results[i, j, 2] <- mean(surplus[i,j,]) / ref.price
      }

  dimnames(results) <- list(dimnames(budgets)[[1]], dimnames(budgets)[[2]], c("Median", "Mean"))

  results
}

surplus.multicomm <- function(budgets.list, financing.needs.list, n.sims.trunc = dim(financing.needs[[1]])[3], r = "0.05", n = "5")
{
  comms <- sort(names(budgets.list))

  probs <- c(0.5, 0.9, 0.95, 0.99)

  results <- array(0, c(length(comms), length(probs)+1))
  dimnames(results) <- list(comms, c("D(0.50)", "D(0.90)", "D(0.95)", "D(0.99)", "Upp. Bd."))

  for (comm in comms)
    {
      ref.price <- get.ref.price.budgets(budgets.list[[comm]])
      price.quants <- quantile(budgets.list[[comm]][r, n, 1:n.sims.trunc], probs)
      price.quants <- c(price.quants, ref.price * K.R(as.numeric(n), 1, as.numeric(r)) / (1 + as.numeric(r)))

      surplus <- array(0, c(length(probs)+1, n.sims.trunc))

      budgets <- budgets.list[[comm]][r, n, 1:n.sims.trunc]
      financing.needs <- financing.needs.list[[comm]][r, n, 1:n.sims.trunc]

      for (s in 1:n.sims.trunc)
        {
          x <- (budgets[s] < price.quants)
          y <- price.quants - financing.needs[s]
          surplus[,s] <- pmax(x * y, 0)
        }
      for (j in 1:(length(probs)+1))
        results[comm, j] <- median(surplus[j, ]) / ref.price
    }

  results
}

financing.needs <- list()

for (comm in names(sims))
{
  cat(comm, fill = TRUE)
  data(list = sims[[comm]])
  results <- financing.needs.sims(eval(as.symbol(sims[[comm]])))
  rm(list = sims[[comm]])
  financing.needs[[comm]] <- results
}

save(financing.needs, file="data/financing.needs.RData")

## Generates table 3.5 page 25 of Villemot (2004)
data(budgets.sym)
print(budgets.table.multicomm(budgets.sym)[,-1,1] - surplus.multicomm(budgets.sym, financing.needs))
