## Computes volatility indices for commodity prices
## Produces table 1.2 (p. 6) of Villemot (2004)

source("common.R")

volatility.indices <- function(comm)
{
  x <- diff(get.log.annual.data(comm))
  c(mean(x), sd(x), variability.indices.split(comm), uncertainty.indices(comm))
}

variability.indices.split <- function(comm)
{
  s <- diff(get.log.annual.data(comm))
  if (start(s)[1] <= 1972)
    x1 <- window(s, start = 1958, end = 1972)
  else
    x1 <- NULL

  x2 <- window(s, start = 1973, end = 1984)
  x3 <- window(s, start = 1985, end = 2003)
  c(if (is.null(x1)) NA else sd(x1), sd(x2), sd(x3))
}

uncertainty.indices <- function(comm)
{
  s <- get.log.annual.data(comm)
  n <- length(s)

  Y <- diff(s)[2:(n-1)]
  t <- 1:(n-2)
  t2 <- t^2
  X1 <- diff(s)[1:(n-2)]
  X2 <- s[1:(n-2)]

  resid <- lm(Y ~ 1 + t + t2 + X1 + X2)$residuals

  sd <- garch(resid, trace = FALSE)$fitted.values[, "sigt"]

  uncert <- mean(sd[2:(n-2)])

  uncert.1985.2003 <- mean(sd[(n-20):(n-2)])

  if (n > 32)
    uncert.1973.1984 <- mean(sd[(n-32):(n-21)])
  else
    uncert.1973.1984 <- mean(sd[2:(n-21)])

  if (n > 32)
    uncert.1957.1972 <- mean(sd[(n-45):(n-33)])	# On perd la sd pour 1957 et 1958 à cause du lag d'ordre 2 dans la régression, et 1959 à cause de l'AR(1) dans le GARCH
  else
    uncert.1957.1972 <- NA

  c(uncert, uncert.1957.1972, uncert.1973.1984, uncert.1985.2003)
}

s <- numeric()
for (comm in names(comm.names))
  {
    s <- cbind(s, volatility.indices(comm))
  }

s <- formatC(s, digits = 3, format = "f")

cat(as.character(comm.names), sep = " & ")
cat(" \\\\ \\hline", fill = TRUE)

for (i in 1:nrow(s))
  {
    cat(s[i, ], sep = " & ")
    cat(" \\\\", fill = TRUE)
  }
